#!/usr/bin/env python
# -*- coding: utf-8 -*-
###############################################################################
# (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
# =============================================================================
## @file TestFONLL.py
#
#  tests conversion of FONLL format to graphs 
#
#  @author Vanya BELYAEV Ivan.Belyaev@itep.ru
#  @date 2014-05-11
# 
# =============================================================================
""" Tests conversion to FONLL format to graphs 
"""
# =============================================================================
__author__  = "Vanya BELYAEV Ivan.Belyaev@itep.ru"
__date__    = "2015-09-01"
__all__     = ()  ## nothing to be imported 
# =============================================================================
import ROOT
import Ostap.PyRoUts
# =============================================================================
# logging 
# =============================================================================
from AnalysisPython.Logger import getLogger
if '__main__' == __name__  or '__builtin__'  == __name__ : 
    logger = getLogger ( 'Ostap.TestFONLL' )
else : 
    logger = getLogger ( __name__ )
# =============================================================================
logger.info ( 'Test converisons from   FONLL format')
# =============================================================================
from Ostap.GraphDeco import ( makeGraph2  , makeGraph3  ,
                              makeGraphs3 , makeGraphs4 )


data = """
# Job started on: Tue Sep  1 10:34:37 CEST 2015 ........
# FONLL heavy quark hadroproduction cross section
# FONLL version and perturbative order: ## FONLL v1.3.2 fonll [ds/dpt^2dy (pb/GeV^2)]
# quark = charm
# final state = meson (meson = D0). NP params (cm,lm,hm) = 0.1, 0.06, 0.135
# BR(q->meson) = 1
# ebeam1 = 3500, ebeam2 = 3500
# PDF set = CTEQ6.6
# ptmin = 0
# ptmax = 20
# ymin  = 2
# ymax  = 4.5
# Uncertainties from scales, masses, PDFs combined quadratically
# cross section is ds/dpt (pb/GeV)
#  pt      central      min       max       min_sc     max_sc     min_mass   max_mass     min_pdf    max_pdf
   0.0000 0.0000e+00 0.0000e+00 0.0000e+00 0.0000e+00 0.0000e+00 0.0000e+00 0.0000e+00 0.0000e+00 0.0000e+00
   0.2500 1.6222e+08 6.4164e+07 5.8307e+08 8.1206e+07 5.8023e+08 1.3633e+08 1.6222e+08 1.1342e+08 2.1102e+08
   0.5000 2.7473e+08 1.1936e+08 8.4278e+08 1.4364e+08 8.3786e+08 2.3737e+08 2.7473e+08 2.0018e+08 3.4928e+08
   0.7500 3.6066e+08 1.1625e+08 1.0107e+09 1.4249e+08 1.0024e+09 2.9168e+08 4.1797e+08 2.7475e+08 4.4657e+08
   1.0000 4.1151e+08 9.3992e+07 1.0863e+09 1.2054e+08 1.0711e+09 3.1762e+08 5.2540e+08 3.2585e+08 4.9719e+08
   1.2500 4.1927e+08 7.5408e+07 1.0539e+09 1.0027e+08 1.0335e+09 3.1662e+08 5.5942e+08 3.4219e+08 4.9636e+08
   1.5000 3.9192e+08 6.3347e+07 9.4105e+08 8.4751e+07 9.1948e+08 2.9478e+08 5.2994e+08 3.2734e+08 4.5650e+08
   1.7500 3.4519e+08 5.9155e+07 7.9363e+08 7.6720e+07 7.7395e+08 2.6113e+08 4.6600e+08 2.9349e+08 3.9689e+08
   2.0000 2.9256e+08 6.2884e+07 6.4601e+08 7.7226e+07 6.2939e+08 2.2358e+08 3.9177e+08 2.5224e+08 3.3287e+08
   2.2500 2.4219e+08 6.6487e+07 5.1533e+08 7.8195e+07 5.0189e+08 1.8727e+08 3.2093e+08 2.1118e+08 2.7320e+08
   2.5000 1.9781e+08 6.5094e+07 4.0694e+08 7.4526e+07 3.9630e+08 1.5479e+08 2.5924e+08 1.7408e+08 2.2155e+08
   2.7500 1.6048e+08 5.9974e+07 3.2016e+08 6.7471e+07 3.1182e+08 1.2700e+08 2.0805e+08 1.4231e+08 1.7865e+08
   3.0000 1.2986e+08 5.3669e+07 2.5194e+08 5.9622e+07 2.4542e+08 1.0383e+08 1.6665e+08 1.1591e+08 1.4380e+08
   3.2500 1.0505e+08 4.7176e+07 1.9874e+08 5.1912e+07 1.9364e+08 8.4793e+07 1.3357e+08 9.4296e+07 1.1580e+08
   3.5000 8.5100e+07 4.0967e+07 1.5731e+08 4.4749e+07 1.5331e+08 6.9286e+07 1.0729e+08 7.6764e+07 9.3436e+07
   3.7500 6.9106e+07 3.5329e+07 1.2496e+08 3.8363e+07 1.2180e+08 5.6716e+07 8.6421e+07 6.2604e+07 7.5605e+07
   4.0000 5.6325e+07 3.0322e+07 9.9842e+07 3.2767e+07 9.7354e+07 4.6573e+07 6.9901e+07 5.1217e+07 6.1430e+07
   4.2500 4.6070e+07 2.5961e+07 8.0188e+07 2.7935e+07 7.8222e+07 3.8376e+07 5.6748e+07 4.2035e+07 5.0103e+07
   4.5000 3.7796e+07 2.2208e+07 6.4650e+07 2.3800e+07 6.3098e+07 3.1723e+07 4.6206e+07 3.4598e+07 4.0995e+07
   4.7500 3.1183e+07 1.9002e+07 5.2468e+07 2.0295e+07 5.1236e+07 2.6355e+07 3.7847e+07 2.8627e+07 3.3740e+07
   5.0000 2.5875e+07 1.6272e+07 4.2878e+07 1.7331e+07 4.1893e+07 2.2005e+07 3.1194e+07 2.3814e+07 2.7936e+07
   5.2500 2.1565e+07 1.3949e+07 3.5236e+07 1.4821e+07 3.4444e+07 1.8447e+07 2.5834e+07 1.9892e+07 2.3239e+07
   5.5000 1.8037e+07 1.1887e+07 2.9087e+07 1.2596e+07 2.8449e+07 1.5516e+07 2.1477e+07 1.6672e+07 1.9402e+07
   5.7500 1.5158e+07 1.0096e+07 2.4140e+07 1.0666e+07 2.3623e+07 1.3109e+07 1.7945e+07 1.4038e+07 1.6279e+07
   6.0000 1.2799e+07 8.6098e+06 2.0139e+07 9.0715e+06 1.9718e+07 1.1125e+07 1.5070e+07 1.1875e+07 1.3723e+07
   6.2500 1.0852e+07 7.3687e+06 1.6883e+07 7.7442e+06 1.6537e+07 9.4776e+06 1.2712e+07 1.0085e+07 1.1619e+07
   6.5000 9.2342e+06 6.3254e+06 1.4214e+07 6.6326e+06 1.3930e+07 8.1010e+06 1.0766e+07 8.5951e+06 9.8737e+06
   6.7500 7.8847e+06 5.4454e+06 1.2016e+07 5.6978e+06 1.1781e+07 6.9461e+06 9.1515e+06 7.3491e+06 8.4205e+06
   7.0000 6.7577e+06 4.7039e+06 1.0201e+07 4.9122e+06 1.0005e+07 5.9774e+06 7.8098e+06 6.3068e+06 7.2087e+06
   7.2500 5.8124e+06 4.0765e+06 8.6941e+06 4.2490e+06 8.5307e+06 5.1614e+06 6.6896e+06 5.4312e+06 6.1937e+06
   7.5000 5.0160e+06 3.5433e+06 7.4375e+06 3.6867e+06 7.3006e+06 4.4708e+06 5.7503e+06 4.6924e+06 5.3396e+06
   7.7500 4.3420e+06 3.0881e+06 6.3849e+06 3.2079e+06 6.2698e+06 3.8839e+06 4.9592e+06 4.0662e+06 4.6180e+06
   8.0000 3.7695e+06 2.6981e+06 5.4996e+06 2.7986e+06 5.4023e+06 3.3830e+06 4.2904e+06 3.5334e+06 4.0056e+06
   8.2500 3.2819e+06 2.3634e+06 4.7526e+06 2.4480e+06 4.6701e+06 2.9548e+06 3.7232e+06 3.0792e+06 3.4848e+06
   8.5000 2.8659e+06 2.0759e+06 4.1205e+06 2.1473e+06 4.0503e+06 2.5881e+06 3.2411e+06 2.6911e+06 3.0408e+06
   8.7500 2.5096e+06 1.8280e+06 3.5835e+06 1.8885e+06 3.5234e+06 2.2729e+06 2.8297e+06 2.3584e+06 2.6609e+06
   9.0000 2.2035e+06 1.6135e+06 3.1256e+06 1.6651e+06 3.0741e+06 2.0012e+06 2.4774e+06 2.0722e+06 2.3348e+06
   9.2500 1.9395e+06 1.4275e+06 2.7338e+06 1.4715e+06 2.6895e+06 1.7661e+06 2.1748e+06 1.8252e+06 2.0538e+06
   9.5000 1.7112e+06 1.2656e+06 2.3974e+06 1.3033e+06 2.3592e+06 1.5622e+06 1.9139e+06 1.6114e+06 1.8110e+06
   9.7500 1.5132e+06 1.1244e+06 2.1078e+06 1.1568e+06 2.0746e+06 1.3847e+06 1.6883e+06 1.4258e+06 1.6006e+06
  10.0000 1.3412e+06 1.0009e+06 1.8578e+06 1.0289e+06 1.8290e+06 1.2301e+06 1.4929e+06 1.2644e+06 1.4180e+06
  10.2500 1.1914e+06 8.9290e+05 1.6415e+06 9.1709e+05 1.6164e+06 1.0950e+06 1.3233e+06 1.1237e+06 1.2590e+06
  10.5000 1.0606e+06 7.9811e+05 1.4538e+06 8.1910e+05 1.4318e+06 9.7683e+05 1.1755e+06 1.0008e+06 1.1203e+06
  10.7500 9.4606e+05 7.1474e+05 1.2904e+06 7.3301e+05 1.2712e+06 8.7311e+05 1.0466e+06 8.9317e+05 9.9895e+05
  11.0000 8.4560e+05 6.4124e+05 1.1479e+06 6.5719e+05 1.1310e+06 7.8188e+05 9.3364e+05 7.9866e+05 8.9253e+05
  11.2500 7.5724e+05 5.7631e+05 1.0233e+06 5.9027e+05 1.0083e+06 7.0144e+05 8.3457e+05 7.1548e+05 7.9899e+05
  11.5000 6.7934e+05 5.1882e+05 9.1395e+05 5.3107e+05 9.0074e+05 6.3038e+05 7.4743e+05 6.4211e+05 7.1656e+05
  11.7500 6.1053e+05 4.6782e+05 8.1788e+05 4.7860e+05 8.0618e+05 5.6746e+05 6.7062e+05 5.7726e+05 6.4378e+05
  12.0000 5.4963e+05 4.2250e+05 7.3328e+05 4.3202e+05 7.2288e+05 5.1166e+05 6.0277e+05 5.1983e+05 5.7940e+05
  12.2500 4.9564e+05 3.8217e+05 6.5865e+05 3.9060e+05 6.4939e+05 4.6210e+05 5.4276e+05 4.6891e+05 5.2236e+05
  12.5000 4.4770e+05 3.4623e+05 5.9268e+05 3.5370e+05 5.8441e+05 4.1800e+05 4.8956e+05 4.2366e+05 4.7173e+05
  12.7500 4.0504e+05 3.1414e+05 5.3423e+05 3.2078e+05 5.2682e+05 3.7870e+05 4.4231e+05 3.8337e+05 4.2669e+05
  13.0000 3.6700e+05 2.8542e+05 4.8234e+05 2.9134e+05 4.7570e+05 3.4359e+05 4.0025e+05 3.4744e+05 3.8655e+05
  13.2500 3.3303e+05 2.5970e+05 4.3619e+05 2.6498e+05 4.3022e+05 3.1218e+05 3.6275e+05 3.1534e+05 3.5072e+05
  13.5000 3.0264e+05 2.3660e+05 3.9507e+05 2.4134e+05 3.8969e+05 2.8404e+05 3.2925e+05 2.8662e+05 3.1866e+05
  13.7500 2.7540e+05 2.1584e+05 3.5836e+05 2.2009e+05 3.5350e+05 2.5878e+05 2.9928e+05 2.6086e+05 2.8994e+05
  14.0000 2.5095e+05 1.9715e+05 3.2553e+05 2.0097e+05 3.2114e+05 2.3606e+05 2.7241e+05 2.3774e+05 2.6416e+05
  14.2500 2.2897e+05 1.8030e+05 2.9612e+05 1.8374e+05 2.9215e+05 2.1562e+05 2.4829e+05 2.1695e+05 2.4099e+05
  14.5000 2.0918e+05 1.6508e+05 2.6974e+05 1.6818e+05 2.6614e+05 1.9718e+05 2.2660e+05 1.9822e+05 2.2013e+05
  14.7500 1.9134e+05 1.5133e+05 2.4604e+05 1.5413e+05 2.4276e+05 1.8054e+05 2.0708e+05 1.8134e+05 2.0134e+05
  15.0000 1.7524e+05 1.3888e+05 2.2471e+05 1.4142e+05 2.2173e+05 1.6550e+05 1.8947e+05 1.6609e+05 1.8438e+05
  15.2500 1.6068e+05 1.2760e+05 2.0550e+05 1.2991e+05 2.0278e+05 1.5189e+05 1.7358e+05 1.5230e+05 1.6905e+05
  15.5000 1.4750e+05 1.1737e+05 1.8816e+05 1.1947e+05 1.8567e+05 1.3956e+05 1.5920e+05 1.3982e+05 1.5518e+05
  15.7500 1.3555e+05 1.0807e+05 1.7248e+05 1.0998e+05 1.7021e+05 1.2836e+05 1.4618e+05 1.2849e+05 1.4261e+05
  16.0000 1.2470e+05 9.9605e+04 1.5830e+05 1.0135e+05 1.5621e+05 1.1819e+05 1.3438e+05 1.1821e+05 1.3120e+05
  16.2500 1.1485e+05 9.1898e+04 1.4545e+05 9.3494e+04 1.4353e+05 1.0894e+05 1.2367e+05 1.0887e+05 1.2083e+05
  16.5000 1.0588e+05 8.4869e+04 1.3378e+05 8.6331e+04 1.3202e+05 1.0050e+05 1.1393e+05 1.0037e+05 1.1140e+05
  16.7500 9.7710e+04 7.8451e+04 1.2319e+05 7.9792e+04 1.2156e+05 9.2817e+04 1.0506e+05 9.2619e+04 1.0281e+05
  17.0000 9.0259e+04 7.2586e+04 1.1355e+05 7.3818e+04 1.1205e+05 8.5798e+04 9.6985e+04 8.5553e+04 9.4973e+04
  17.2500 8.3456e+04 6.7219e+04 1.0477e+05 6.8353e+04 1.0339e+05 7.9383e+04 8.9618e+04 7.9101e+04 8.7818e+04
  17.5000 7.7240e+04 6.2303e+04 9.6771e+04 6.3349e+04 9.5494e+04 7.3515e+04 8.2890e+04 7.3205e+04 8.1278e+04
  17.7500 7.1551e+04 5.7796e+04 8.9467e+04 5.8762e+04 8.8285e+04 6.8140e+04 7.6740e+04 6.7810e+04 7.5293e+04
  18.0000 6.6340e+04 5.3662e+04 8.2791e+04 5.4555e+04 8.1697e+04 6.3214e+04 7.1110e+04 6.2866e+04 6.9812e+04
  18.2500 6.1562e+04 4.9865e+04 7.6684e+04 5.0691e+04 7.5669e+04 5.8693e+04 6.5950e+04 5.8332e+04 6.4788e+04
  18.5000 5.7175e+04 4.6373e+04 7.1090e+04 4.7140e+04 7.0147e+04 5.4541e+04 6.1217e+04 5.4169e+04 6.0176e+04
  18.7500 5.3145e+04 4.3161e+04 6.5961e+04 4.3873e+04 6.5084e+04 5.0724e+04 5.6871e+04 5.0344e+04 5.5940e+04
  19.0000 4.9438e+04 4.0202e+04 6.1255e+04 4.0864e+04 6.0438e+04 4.7211e+04 5.2877e+04 4.6825e+04 5.2045e+04
  19.2500 4.6026e+04 3.7474e+04 5.6931e+04 3.8090e+04 5.6170e+04 4.3975e+04 4.9203e+04 4.3586e+04 4.8459e+04
  19.5000 4.2882e+04 3.4957e+04 5.2956e+04 3.5531e+04 5.2246e+04 4.0991e+04 4.5819e+04 4.0601e+04 4.5156e+04
  19.7500 3.9982e+04 3.2632e+04 4.9298e+04 3.3168e+04 4.8634e+04 3.8238e+04 4.2702e+04 3.7849e+04 4.2110e+04
  20.0000 3.7307e+04 3.0483e+04 4.5928e+04 3.0984e+04 4.5307e+04 3.5696e+04 3.9826e+04 3.5310e+04 3.9299e+04
  """


gr = makeGraph2 ( data )

gtot,gscale,gmass,gpdf = makeGraphs4(data)

gtot  .draw ( ' ae1p      ' )
gscale.draw ( '  e1p same ' )
gmass .draw ( '  e1p same ' )
gpdf  .draw ( '  e1p same ' )

gscale.red   ()
gmass .blue  ()
gpdf  .green ()

import warnings 
with warnings.catch_warnings():
    warnings.simplefilter("ignore")
    from  Ostap.Utils import mute
    with mute() : 
        i8_20 = gtot.integral ( 8.0 , 20.0 )
        i0_20 = gtot.integral ( 0.0 , 20.0 )
    
logger.info ( 'Integrals:  I(8,20)/I(0,20) = %.3g%%' % (100.*i8_20/i0_20)  )

# =============================================================================
# The END
# =============================================================================
