#!/usr/bin/env gaudirun.py
###############################################################################
# (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
# =============================================================================
# =============================================================================
## @file
#  The configuration file to run CheckMC_Bs2PsiPhi example
#
#   This file is a part of LoKi project - 
#     "C++ ToolKit  for Smart and Friendly Physics Analysis"
#
#  The package has been designed with the kind help from
#  Galina PAKHLOVA and Sergey BARSUK.  Many bright ideas, 
#  contributions and advices from G.Raven, J.van Tilburg, 
#  A.Golutvin, P.Koppenburg have been used in the design.
#
#
#  @author Vanya BELYAEV Ivan.Belyaev@nikhef.nl
#  @date   2008-06-07
# =============================================================================
"""
The configuration file to run CheckMC_Bs2PsiPhi example

This file is a part of LoKi project - 
\"C++ ToolKit  for Smart and Friendly Physics Analysis\"

The package has been designed with the kind help from
Galina PAKHLOVA and Sergey BARSUK.  Many bright ideas, 
contributions and advices from G.Raven, J.van Tilburg, 
A.Golutvin, P.Koppenburg have been used in the design.


"""
# =============================================================================
__author__  = " Vanya BELYAEV Ivan.Belyaev@nikhef.nl "
# =============================================================================
import os 
from  Gaudi.Configuration import * 
from  Configurables import PhysDesktop
from  Configurables import LoKi__Bs2PsiPhi         as PsiPhi
from  Configurables import LoKi__CheckMC_Bs2PsiPhi as CheckMC

importOptions( "$DAVINCIROOT/options/DaVinciCommon.opts" )
importOptions( "$COMMONPARTICLESROOT/options/StandardKaons.opts" )
importOptions( "$COMMONPARTICLESROOT/options/StandardMuons.opts" )

## confgure the application itself:
appMgr = ApplicationMgr( EvtMax = 1000 )

## configure our own algorithm: 
alg = PsiPhi('PsiPhi')
alg.addTool ( PhysDesktop() )
alg.PhysDesktop.InputLocations = ['Phys/StdTightKaons' , 'Phys/StdTightMuons']
alg.PP2MCs = [ 'Relations/Rec/ProtoP/Charged' ]

appMgr.TopAlg += [ alg ]

## configure MC Check
check = CheckMC ( "CheckMC" )
check.addTool ( PhysDesktop() )
check.PhysDesktop.InputLocations = ['Phys/PsiPhi']
check.PP2MCs = [ 'Relations/Rec/ProtoP/Charged' ]

appMgr.TopAlg += [ check ]

## histograms:
HistogramPersistencySvc ( OutputFile = "CheckMC_Histos.root" )

## input data:
from LoKiExample.Bs2Jpsiphi_mm_data import Inputs
EventSelector ( Input     = Inputs ,
                PrintFreq = 20     ) 

# =============================================================================
# The END
# =============================================================================
