/*****************************************************************************\
* (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
// ============================================================================
// Include files 
// ============================================================================
// AnalysisPython
// ============================================================================
#include "Analysis/PySelectorWithCuts.h"
#include "Analysis/Formula.h"
// ============================================================================
// ROOT 
// ============================================================================
#include "TCut.h"
// ============================================================================
/** @file 
 *  Implementation file for class Analysis::SelectorWithCuts
 * 
 *  @date 2013-05-06 
 *  @author Vanya BELYAEV Ivan.Belyaev@itep.ru
 *
 */
// ============================================================================
// constructor 
// ============================================================================
Analysis::SelectorWithCuts::SelectorWithCuts
( const std::string& cuts , 
  TTree*             tree , 
  PyObject*          self ) 
  : Analysis::Selector ( tree , self ) 
  , fMycuts            ( cuts        ) 
  , fMyformula         ( 0           ) 
  , m_event            ( 0           )            
{
  if ( 0 != tree ) 
  { fMyformula = new Analysis::Formula ( "" , fMycuts , tree ) ; }
}
// ============================================================================
// constructor 
// ============================================================================
Analysis::SelectorWithCuts::SelectorWithCuts
( const TCut&        cuts , 
  TTree*             tree , 
  PyObject*          self ) 
  : Analysis::Selector ( tree , self      ) 
  , fMycuts            ( cuts.GetTitle () ) 
  , fMyformula         ( 0           ) 
  , m_event            ( 0           )            
{
  if ( 0 != tree ) 
  { fMyformula = new Analysis::Formula ( "" , fMycuts , tree ) ; }
}
// ============================================================================
// virtual destructor 
// ============================================================================
Analysis::SelectorWithCuts::~SelectorWithCuts ()
{ if ( 0 != fMyformula ) { delete fMyformula ; fMyformula = 0 ; } }
// ============================================================================
// notify 
// ============================================================================
Bool_t Analysis::SelectorWithCuts::Notify() 
{
  if ( 0 != fMyformula ) { fMyformula->Notify() ; }
  return TPySelector::Notify () ;
}
// ============================================================================
// init 
// ============================================================================
void Analysis::SelectorWithCuts::Init ( TTree* tree ) 
{
  /// reset the event counter 
  m_event = 0 ;
  //
  if ( 0 != fMyformula ) { delete fMyformula ; fMyformula = 0 ; }
  fMyformula = new Analysis::Formula ( "" , fMycuts , tree ) ;
  //
  TPySelector::Init ( tree ) ;
}
// ============================================================================
//  begin 
// ============================================================================
void Analysis::SelectorWithCuts::Begin ( TTree* tree ) 
{
  TPySelector::Begin ( tree ) ;
}
// ============================================================================
// slave begin 
// ============================================================================
void Analysis::SelectorWithCuts::SlaveBegin ( TTree* tree ) 
{
  //
  if ( 0 != fMyformula ) { delete fMyformula ; fMyformula = 0 ; }
  fMyformula = new Analysis::Formula ( "" , fMycuts , tree ) ;
  //
  TPySelector::SlaveBegin ( tree ) ;
}
// ============================================================================
// process 
// ============================================================================
Bool_t Analysis::SelectorWithCuts::Process      ( Long64_t entry ) 
{
  /// increment the event counter 
  ++m_event  ;
  //
  if ( 0 != fMyformula && fMyformula->GetNdim() && !fMyformula ->evaluate() ) 
  { return false ; }
  //
  return TPySelector::Process ( entry ) ;
}
// ============================================================================
// is formula OK?
// ============================================================================
bool Analysis::SelectorWithCuts::ok () const // is formula OK ? 
{ return 0 != fMyformula && fMyformula->ok () ; }


// ============================================================================
// The END 
// ============================================================================
