/*****************************************************************************\
* (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#ifndef P2MCPFROMPROTOP_H
#define P2MCPFROMPROTOP_H 1

// Include files
// from Gaudi
#include "GaudiAlg/GaudiTool.h"
#include "Kernel/IParticle2MCWeightedAssociator.h"            // Interface
#include "Kernel/Particle2MCAssociatorBase.h"
// from LoKi
#include "LoKi/PhysMCTypes.h"

/** @class P2MCPFromProtoP P2MCPFromProtoP.h
 *
 *  ProtoParticle to MC association tool
 *
 *  @author Juan PALACIOS
 *  @date   2009-03-31
 */
class P2MCPFromProtoP : public Particle2MCAssociatorBase
{

public:

  /// Standard constructor
  P2MCPFromProtoP( const std::string& type,
                   const std::string& name,
                   const IInterface* parent );

  virtual ~P2MCPFromProtoP( ); ///< Destructor

  StatusCode initialize() override;

private :

  typedef std::vector<std::string> Addresses ;
  typedef std::vector<const LoKi::Types::TablePP2MC*> Tables;

  Particle2MCParticle::ToVector
  relatedMCPsImpl(const LHCb::Particle* particle,
          const LHCb::MCParticle::ConstVector& mcParticles) const override;

  void loadTables() const;

  inline const Tables& i_tables() const
  {
    return m_tables;
  }

private:

  Addresses m_PP2MC; ///< standard addresses (without /Event)

  mutable Tables m_tables;

};
#endif // P2MCPFROMPROTOP_H
