###############################################################################
# (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
__author__  = "Victor Coco <Victor.Coco@cern.ch>"

from LHCbKernel.Configuration import *

from Configurables import ( GaudiSequencer, LoKi__MCJetMaker,  LoKi__HEPMCJetMaker, LoKi__FastJetMaker, HistogramSvc )

class HEPMCJetMakerConf:

    def __init__(self, _name, PartTypes = ['pi+','K+','p+','mu+','e+','gamma','pi0','KS0','KL0','n0','f_0(980)','f_0(1370)'],
                 R = 0.5 ,
                 PtMin = 5000.,
                 MotherCut = "GNONE" ,
                 ToBanCut = "GNONE" ,
                 SimpleAcceptance = False ,
                 jetidnumber = 999 ,
                 algtype="anti-kt",
                 OutputTable = "",
                 SaveMotherOnly = False ):


        jetname_dict = {"kt":0,"Cambridge":1,"anti-kt":2}
        self.name = _name
        self.R = R
        self.SimpleAcceptance = SimpleAcceptance
        self.CodeForMotherSelection = MotherCut
        self.CodeForBannedSelection = ToBanCut
        self.PtMin = PtMin
        self.jetMakerType =  jetname_dict[algtype]
        self.jetidnumber = jetidnumber
        self.MCParticleTypes = PartTypes
        self.SaveMotherOnly = SaveMotherOnly
        self.OutputTable = OutputTable
        self.algorithms = []
        self.setupHEPJetMaker()

    def setupHEPJetMaker(self):
        jetMakerName = self.name
        algo =  LoKi__HEPMCJetMaker ( jetMakerName )
        algo.CodeForMotherSelection = self.CodeForMotherSelection
        algo.CodeForBannedSelection = self.CodeForBannedSelection
        algo.OutputTable = self.OutputTable
        algo.SaveMCJetsFromMotherOnly = self.SaveMotherOnly
        algo.SimpleAcceptance = self.SimpleAcceptance
        nametool = LoKi__FastJetMaker.__name__
        algo.addTool(LoKi__FastJetMaker, name=nametool)
        tool = getattr ( algo , nametool )
        algo.JetMaker = tool.getTitleName()
        tool.Type = self.jetMakerType
        tool.RParameter = self.R
        tool.PtMin = self.PtMin
        tool.Recombination = 0
        tool.JetID = self.jetidnumber
        self.algorithms.append(algo)




class MCJetMakerConf:

    def __init__(self, _name,
                 R = 0.5 ,
                 PtMin = 5000.,
                 MotherCut = "MCNONE" ,
                 ToBanCut = "MCNONE" ,
                 SimpleAcceptance = False ,
                 jetidnumber = 99 ,
                 algtype="anti-kt",
                 OutputTableMC = "",
                 SaveMotherOnly = False ):


        jetname_dict = {"kt":0,"Cambridge":1,"anti-kt":2}
        self.name = _name
        self.R = R
        self.SimpleAcceptance = SimpleAcceptance
        self.CodeForMotherSelection = MotherCut
        self.CodeForBannedSelection = ToBanCut
        self.PtMin = PtMin
        self.jetMakerType =  jetname_dict[algtype]
        self.jetidnumber = jetidnumber
        self.SaveMotherOnly = SaveMotherOnly
        self.OutputTableMC = OutputTableMC
        self.algorithms = []
        self.setupMCJetMaker()


    def setupMCJetMaker(self):
        from GaudiKernel.ProcessJobOptions import importOptions
        jetMakerName = self.name
        algo =  LoKi__MCJetMaker ( jetMakerName )
        algo.CodeForMotherSelectionMC = self.CodeForMotherSelection
        algo.CodeForBannedSelectionMC = self.CodeForBannedSelection
        algo.OutputTableMC = self.OutputTableMC
        algo.SaveMCJetsFromMotherOnly = self.SaveMotherOnly
        algo.SimpleAcceptance = self.SimpleAcceptance
        nametool = LoKi__FastJetMaker.__name__
        algo.addTool(LoKi__FastJetMaker, name=nametool)
        tool = getattr ( algo , nametool )
        algo.JetMaker = tool.getTitleName()
        tool.Type = self.jetMakerType
        tool.RParameter = self.R
        tool.PtMin = self.PtMin
        tool.Recombination = 0
        tool.JetID = self.jetidnumber
        self.algorithms.append(algo)
