#!/usr/bin/env python
# -*- coding: utf-8 -*-
###############################################################################
# (c) Copyright 2000-2019 CERN for the benefit of the LHCb Collaboration      #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
# =========================================================================================
# =========================================================================================
## @file
#  simple examples how to build dataset from TChain/Tree
#  - book and define variables/columns
#  - define filter criteria
#  This is not the only  way to build dataset (n particular it is rather slow),
#  but it has certain advantages  
#  @see RooDataSet
#  @author Vanya BELYAEV  Ivan.Belyaev@itep.ru
#
# =========================================================================================
""" Simple example  to fill dataset for TChain/TTree
- book and define variables/columns
- define filter criteria  
This is not the only  way to build dataset  (in particular it is rather slow),
but it has certain advantages  
"""
# =========================================================================================
import ROOT
from   Ostap.PyRoUts         import * 
from   AnalysisPython.Logger import getLogger
from   Ostap.Selectors       import SelectorWithVars 
# ==========================================================================================
if '__main__' == __name__ : logger = getLogger('OstapTutor/data2')
else                      : logger = getLogger( __name__         )
# ==========================================================================================

from OstapTutor.TestVars1  import m_psi,m_ups 

## define variables to be used in data set 
variables = [
    ( m_psi     , lambda s : min(s.mass1,s.mass2) ) ,  ## use pre-existing RootRealVar
    ( m_ups     , lambda s : max(s.mass1,s.mass2) ) ,  ## ditto 
    ( "mass"    , "mass(4mu)" , 0 , 1e+6  , lambda s : s.mass ) , ## make new RooRealVar
    ]

## get TChain
import data1 as     data 
chain = data.data_YY.chain

## get basic cuts:
from ycuts   import cuts

## make selector 
sel_Ypsi = SelectorWithVars (
    variables ,
    selection = cuts 
    )

## fill selector 
chain.process ( sel_Ypsi )

## get dataset
ds_Ypsi = sel_Ypsi.data

print ds_Ypsi  

# ==========================================================================================
# The END 
# ==========================================================================================
